@@ -14,6 +14,9 @@
 //                                                            ///
 /////////////////////////////////////////////////////////////////
 
+if (!defined('GETID3_INCLUDEPATH')) { // prevent path-exposing attacks that access modules directly on public webservers
+	exit;
+}
 
 class getid3_tiff extends getid3_handler
 {
@@ -36,7 +39,6 @@ class getid3_tiff extends getid3_handler
 			default:
 				$this->error('Invalid TIFF byte order identifier ('.substr($TIFFheader, 0, 2).') at offset '.$info['avdataoffset']);
 				return false;
-				break;
 		}
 
 		$info['fileformat']          = 'tiff';
